#include <X11/Xlib.h>
#include <string.h>
#include <beryl.h>
#include <beryl-private.h>
#include <fsp.h>

//this code is thanks to the wonderful work and documentation in kwin

#define FSP(x) (x)->screen->display->focusStealingLevel

static int timestampCompare(Time t1, Time t2)
{
	if (t1 == t2)
		return 0;
	return (t1 - t2) < 0x7fffffffU ? 1 : -1;
}

Bool shouldFocusWindow(CompWindow * w, Time t, Bool focus_in)
{
	// KWIN Focus Stealing Levels:
	// 0 - none, new windows always get focus
	// 1 - low, allow activation if unsure
	// 2 - normal, disallow if not sure
	// 3 - high, only focus if belongs to active app
	// 4 - extreme, only user-initiated focus
	if (t == -1U)
	{
		if (!getWindowUserTime(w, &t))
			t = -1U;
	}
	if (!t)
		return False;
	/*if (focus_in && w->awaitingFocus)
	   {
	   w->awaitingFocus=False;
	   return True;
	   } */
	if (FSP(w) == FSP_NONE)
		return True;
	if (FSP(w) == FSP_EXTREME)
		return False;
	int vx, vy;

	defaultViewportForWindow(w, &vx, &vy);
	if (vx != w->screen->x || vy != w->screen->y)
		return False;

	CompWindow *law = findWindowAtDisplay(w->screen->display,
										  w->screen->display->activeWindow);

	if (!law)
		return True;
	if (law->type & CompWindowTypeDesktopMask)
		return True;
	if (law->clientLeader == w->clientLeader ||
		(law->resClass && w->resClass && !strcmp(law->resClass, w->resClass)))
		return True;
	if (FSP(w) == FSP_HIGH)
		return False;
	if (t == -1U)
	{
		if (FSP(w) == FSP_LOW)
			return True;
		else
			return False;
	}
	Time t2;

	if (!getWindowUserTime(law, &t2))
		t2 = -1U;
	return timestampCompare(t, t2) >= 0;
}
